(* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
ParserBuilder(c)2002 Pat Connors ( pat_connors@hotmail.com )

ParserBuilder is for building recursive descent parsers
using the COCO/R language.

Aside from generating source code, ParserBuilder also provides an
IDE from which you can edit, build and test your parsers.

The only programming language supported by
ParserBuilder 1.0 is Borland's Delphi Object Pascal.

The source code for the COCO/R compiler has been completely based on
COCO/R for TurboPascal by Prof. Pat Terry and Volker Pohlers.
You can find this via http://cs.ru.ac.za/homes/cspt/cocor.htm.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program;
if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+- *)

program ParserBuilder;
{$R *.res}


uses
  SysUtils,
  Forms,
  pbSHLScanner in 'pbSHL\pbSHLScanner.pas',
  pbSHLParser in 'pbSHL\pbSHLParser.pas',
  COCOTreeBuilder in 'pbCocoModel\COCOTreeBuilder.pas',
  pbCocoImportParser in 'pbImport\pbCocoImportParser.pas',
  pbCocoImportScanner in 'pbImport\pbCocoImportScanner.pas',
  pbSHLG in 'pbSHL\pbSHLg.pas',
  CocoFactor in 'pbCocoModel\CocoFactor.pas',
  CocoCharacters in 'pbCocoModel\CocoCharacters.pas',
  CocoTokens in 'pbCocoModel\CocoTokens.pas',
  CocoPragmas in 'pbCocoModel\CocoPragmas.pas',
  TreeEditSync in 'pbCocoModel\TreeEditSync.pas',
  pbCocoFileSystem in 'pbCocoModel\pbCocoFileSystem.pas',
  pbFileSystem in 'pbCocoModel\pbFileSystem.pas',
  CocoImporter in 'pbCocoModel\CocoImporter.pas',
  CocoComments in 'pbCocoModel\CocoComments.pas',
  CompilerObj in 'pbCocoModel\CompilerObj.pas',
  CocoIgnore in 'pbCocoModel\CocoIgnore.pas',
  DelphiDeclarations in 'pbCocoModel\DelphiDeclarations.pas',
  CocoProductions in 'pbCocoModel\CocoProductions.pas',
  CocoProduction in 'pbCocoModel\CocoProduction.pas',
  CocoProductionPath in 'pbCocoModel\CocoProductionPath.pas',
  CocoToken in 'pbCocoModel\CocoToken.pas',
  NewATGWizard_Main in 'Forms\NewATGWizard_Main.pas' {frmNewATGWizard},
  NewATGWizard in 'pbCocoModel\NewATGWizard.pas',
  pbCocoScanner in 'pbCocoLib\pbCocoScanner.pas',
  pbCocoParser in 'pbCocoLib\pbCocoParser.pas',
  pbCocoTable in 'pbCocoLib\pbCocoTable.pas',
  pbCocoSets in 'pbCocoLib\pbCocoSets.pas',
  pbCocoAutomaton in 'pbCocoLib\pbCocoAutomaton.pas',
  pbCocoParserGen in 'pbCocoLib\pbCocoParserGen.pas',
  pbCocoDriverGen in 'pbCocoLib\pbCocoDriverGen.pas',
  pbCocoConstants in 'pbCocoLib\pbCocoConstants.pas',
  EditStreamCallbackHandler in 'pbSHL\EditStreamCallbackHandler.pas',
  pbStrLib in 'pbCocoModel\pbStrLib.pas',
  pbSHRichEdit in 'pbSHL\pbSHRichEdit.pas',
  DataFile in 'P3Lib\DataFile.pas',
  Wizard in 'P3Lib\Wizard.pas',
  Wizard_Types in 'P3Lib\Wizard_Types.pas',
  ParserBuilder_Main in 'Forms\ParserBuilder_Main.pas' {frmParserBuilderMain},
  AboutDlg in 'Forms\AboutDlg.pas' {frmAbout},
  CocoGUITable in 'pbImport\CocoGUITable.pas',
  pbCocoG in 'pbCocoLib\pbCocog.pas',
  DelphiMethodWizardForm in 'Forms\DelphiMethodWizardForm.pas' {frmDelphiMethodWizard},
  CocoGen3 in 'pbCocoLib\CocoGen3.pas',
  ProductionChecker in 'pbImport\ProductionChecker.pas',
  SimpleSyntaxCheckerParser in 'pbImport\SimpleSyntaxCheckerParser.pas',
  SimpleSyntaxCheckerScanner in 'pbImport\SimpleSyntaxCheckerScanner.pas',
  SemActionFormatterScanner in 'pbCocoModel\SemActionFormatterScanner.pas',
  SemActionFormatterParser in 'pbCocoModel\SemActionFormatterParser.pas',
  SemActionFormatterComponent in 'pbCocoModel\SemActionFormatterComponent.pas',
  ParserUnitImportParserComponent in 'pbParserUnitImport\ParserUnitImportParserComponent.pas',
  ParserUnitImportScanner in 'pbParserUnitImport\ParserUnitImportScanner.pas',
  ParserUnitImportParser in 'pbParserUnitImport\ParserUnitImportParser.pas',
  AALCS2 in 'P3Lib\AALCS2.pas',
  ProdCheck2ParserComponent in 'pbCocoModel\ProdCheck2ParserComponent.pas',
  ProdCheck2Parser in 'pbCocoModel\ProdCheck2Parser.pas',
  ProdCheck2Scanner in 'pbCocoModel\ProdCheck2Scanner.pas',
  SimpleSyntaxCheckerG in 'pbImport\SimpleSyntaxCheckerg.pas',
  WSStripperScanner in 'WSStripper\WSStripperScanner.pas',
  WSStripperParser in 'WSStripper\WSStripperParser.pas',
  WSStripperComponent in 'WSStripper\WSStripperComponent.pas';

begin

  Application.Initialize;
  {$IFDEF MEMCHK}
  MemCheckLogFileName := ChangeFileExt(Application.ExeName, '.mck');
  MemChk;
  {$ENDIF MEMCHK}
  Application.Title := 'ParserBuilder for Delphi';
  Application.HelpFile := 'ParserBuilder.chm';

  {$IFDEF SHOWABOUT}
  // only show this form at startup if its the first time
  // the app is running - too annoying otherwise.
  // if app is virgin, there is no ParserBuilderToolbar.cfg file

  if not FileExists('ParserBuilderToolbar.cfg') then
  begin
    with TfrmAbout.Create(nil) do
    begin
      Show;
      Invalidate;
      Repaint;
      Application.CreateForm(TfrmParserBuilderMain, frmParserBuilderMain);
    end;
  end
  else
    Application.CreateForm(TfrmParserBuilderMain, frmParserBuilderMain);
  {$ELSE}
  Application.CreateForm(TfrmParserBuilderMain, frmParserBuilderMain);
  {$ENDIF SHOWABOUT}
  Application.Run;
end.
