{-----------------------------------------------------------------------------
 Unit Name: pbCocoDriverGen

 Author:     Originally Pat Terry (CRC.PAS) , ported to Delphi by Pat Connors,
 Emits the source code of a generated driver program via frame file.
-----------------------------------------------------------------------------}


unit pbCocoDriverGen;

interface

uses
  Windows, Messages, SysUtils, Classes,
  pbCocoTable, pbCocoScanner, pbCocoAutomaton,
  pbStrLib, FileUtil;

type
  TpbCocoDriverGen = class(TObject)
  private
    FCocoTable: TpbCocoTable;
    FCocoScanner: TpbCocoScanner;
    FCocoAutomaton: TpbCocoAutomaton;
    FFrameSize: Integer;
    { Private declarations }
    procedure PutS(s: string);
    procedure SetCocoTable(const Value: TpbCocoTable);
    procedure SetCocoScanner(const Value: TpbCocoScanner);
    procedure SetCocoAutomaton(const Value: TpbCocoAutomaton);
    procedure SetFrameSize(const Value: Integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    err: TStringStream; (* output: error message texts *)
    fram: TStringStream; (* input:  parser frame parser.frm *)
    com: TStringStream; (* output: generated parser *)
    constructor Create;
    destructor Destroy; override;
    procedure WriteDriver;
    property FrameSize: Integer read FFrameSize write SetFrameSize;
  published
    { Published declarations }
    property CocoTable: TpbCocoTable read FCocoTable write SetCocoTable;
    property CocoScanner: TpbCocoScanner read FCocoScanner write SetCocoScanner;
    property CocoAutomaton: TpbCocoAutomaton read FCocoAutomaton write SetCocoAutomaton;
  end;



implementation

constructor TpbCocoDriverGen.Create;
begin
  inherited;
  com := TStringStream.Create(''); (* output: generated parser *)
end;

destructor TpbCocoDriverGen.Destroy;
begin
  if (err <> nil) then
    err.Free;
  if (fram <> nil) then
    fram.Free;
  com.Free;
  inherited;
end;

procedure TpbCocoDriverGen.PutS(s: string);
var
  i: integer;
begin
  for i := 1 to Length(s) do
    if s[i] = '$' then pbWriteLn(com, '')
  else
    pbWrite(com, s[i]);
end;

procedure TpbCocoDriverGen.SetCocoAutomaton(const Value: TpbCocoAutomaton);
begin
  FCocoAutomaton := Value;
end;

procedure TpbCocoDriverGen.SetCocoScanner(const Value: TpbCocoScanner);
begin
  FCocoScanner := Value;
end;

procedure TpbCocoDriverGen.SetCocoTable(const Value: TpbCocoTable);
begin
  FCocoTable := Value;
end;


procedure TpbCocoDriverGen.SetFrameSize(const Value: Integer);
begin
  FFrameSize := Value;
end;

procedure TpbCocoDriverGen.WriteDriver;
var
  LeftMargin, i: integer;
  gn: tgraphnode;
  sn: tsymbolNode;
  gramName, fGramName, fn, errName, CompilerFrame: string;
begin
  FCocoTable.GetNode(FCocoTable.root, gn);
  FCocoTable.GetSym(gn.p1, sn);
  fn := sn.Name;

  if (not CocoTable.hasCompilerFrame) then
    CompilerFrame := Concat(FCocoScanner.directory, '\pbcompiler.frm')
  else 
    CompilerFrame := CocoTable.CompilerFrame;


  try
    fram := LoadStringStream(CompilerFrame);
    CocoAutomaton.FramInPos := 0;
    CocoAutomaton.FrameInSize := fram.Size;
  except
    raise Exception.Create('"' + CompilerFrame + '" not found - aborted.');
  end;

  LeftMargin := 0;
  gramName := sn.Name;

  if (not CocoTable.hasCompilerTarget) then
    fGramName := Concat(FCocoScanner.OutputDirectory, '\' + gramName)
  else 
    fGramName := CocoTable.CompilerTarget + '\' + gramName;

  errName := Concat(fgramName, '.err');
  if (FileExists(errName)) then
  begin
    err := LoadStringStream(errName);
  end
  else
    Err := TStringStream.Create('(* No Error Messages Specified. *)');

  PutS('// Generated by ParserBuilder @ ' + DateTimeToStr(NOW) + '$$');

  for i := 1 to 14 do
  begin
    CocoAutomaton.CopyFramePart('-->Grammar', LeftMargin, fram, com);
    PutS(gramName);
  end;
  CocoAutomaton.CopyFramePart('-->End.', LeftMargin, fram, com);

  fn := Concat(fGramName, 'TestHarness_Main.pas');

  if FileExists(fn) then
    CopyFile(PChar(fn), PChar(ChangeFileExt(fn, '.pas.backup')), nil);

  SaveStringStream(com, fn);

  // write out the DPR file

  Fram.Free;
  Fram := nil;

  if (Com <> nil) then
    com.Free;
  com := nil;
  com := TStringStream.Create(''); (* output: generated parser *)

  CompilerFrame := Concat(FCocoScanner.directory, '\pbCompiler_DPR.frm');
  try
    fram := LoadStringStream(CompilerFrame);
    CocoAutomaton.FramInPos := 0;
    CocoAutomaton.FrameInSize := fram.Size;
  except
    raise Exception.Create('"pbCompiler_DPR.frm" not found - aborted.');
  end;
  PutS('// Generated by ParserBuilder @ ' + DateTimeToStr(NOW) + '$$');
  for i := 1 to 4 do
  begin
    CocoAutomaton.CopyFramePart('-->Grammar', LeftMargin, fram, com);
    PutS(gramName);
  end;
  PutS('TestHarness *)');

  fn := Concat(fGramName, 'TestHarness.dpr');
  if FileExists(fn) then
    CopyFile(PChar(fn), PChar(ChangeFileExt(fn, '.dpr.backup')), nil);


  SaveStringStream(com, fn);

  fn := Concat(fGramName, 'TestHarness_Main.dfm');
  CompilerFrame := Concat(FCocoScanner.directory, '\pbCompiler.form');
  if FileExists(fn) then
    CopyFile(PChar(fn), PChar(ChangeFileExt(fn, '.dfm.backup')), nil);
  CopyFile(CompilerFrame, fn, nil);
end;

end.
