{-----------------------------------------------------------------------------
 Unit Name: CocoPragmas
 Author:    Administrator
Purpose: Container for GUI-model of ATG File - Holds the Pragmas as a block of text.
-----------------------------------------------------------------------------}


unit CocoPragmas;

interface

uses
  Windows, Messages, SysUtils, Classes,
  pbStrLib, CocoTokens, CocoToken, jclSysUtils;

type
  TCocoPragmas = class(TComponent)
  private
    { Private declarations }
    FPragmaList: TList;
    FComments: string;
    FIsEditing: boolean;
    FEditingPragma: string;
    function GetPragmaCount: Integer;
    procedure SetComments(const Value: string);
    procedure SetIsEditing(const Value: boolean);
    procedure SetEditingPragma(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetATG: string;
    function AddPragmaDefinition(const PragmaDef: string): TCocoToken;
    function GetPragmaDefinition(const PragmaTitle: string): TCocoToken;
    function GetPragmaByIndex(const idx: Integer): TCocoToken;
  published
    { Published declarations }
    property PragmaCount: Integer read GetPragmaCount;
    property Comments: string read FComments write SetComments;
    property IsEditing: boolean read FIsEditing write SetIsEditing;
    property EditingPragma: string read FEditingPragma write SetEditingPragma;
  end;

implementation


{ TCocoPragmas }

function TCocoPragmas.AddPragmaDefinition(const PragmaDef: string): TCocoToken;
begin
  Result := TCocoToken.Create(Self);
  fPragmaList.Add(Result);
end;

constructor TCocoPragmas.Create(AOwner: TComponent);
begin
  inherited;
  fPragmaList := TList.Create;
end;

destructor TCocoPragmas.Destroy;
begin
  FreeObjectList(fPragmaList);
  inherited;
end;

function TCocoPragmas.GetATG: string;
var
  idx: integer;
  ct: TCocoToken;
begin
  if not FIsEditing then
  begin
    Result := 'PRAGMAS' + CRLF;
    for idx := 0 to fPragmaList.Count - 1 do
    begin
      ct := fPragmaList[idx];
      if ct.Symbol <> FEditingPragma then
        Result := Result + ct.GetATG
      else
        Result := Result + ModTextTag + CRLF;
    end;
  end
  else
  begin
    // Editing entire Pragma section
    Result := 'PRAGMAS' + CRLF + ModTextTag + CRLF;
  end;
end;

function TCocoPragmas.GetPragmaByIndex(const idx: Integer): TCocoToken;
begin
  Result := fPragmaList[idx];
end;

function TCocoPragmas.GetPragmaCount: Integer;
begin
  Result := fPragmaList.Count;
end;

function TCocoPragmas.GetPragmaDefinition(const PragmaTitle: string): TCocoToken;
var
  idx: integer;
  ct: TCocoToken;
begin
  Result := nil;
  for idx := 0 to fPragmaList.Count - 1 do
  begin
    ct := fPragmaList[idx];
    if (ct.Symbol = PragmaTitle) then
    begin
      Result := ct;
      Exit;
    end;
  end;
end;

procedure TCocoPragmas.SetComments(const Value: string);
begin
  FComments := Value;
end;

procedure TCocoPragmas.SetEditingPragma(const Value: string);
begin
  FEditingPragma := Value;
end;

procedure TCocoPragmas.SetIsEditing(const Value: boolean);
begin
  FIsEditing := Value;
end;

end.
