{-----------------------------------------------------------------------------
 Unit Name: CocoProduction
 Author:     pat connors
Purpose: Container for GUI-model of ATG File - Holds the individual productions
-----------------------------------------------------------------------------}


unit CocoProduction;

interface

uses
  Windows, Messages, SysUtils, Classes, CocoProductionPath,
  pbStrLib, DataFile, jclSysUtils, SemActionFormatterComponent;

type
  TCocoProduction = class(TComponent)
  private
    { Private declarations }
    FProductionName: string;
    FProductionPaths: TList;
    FProdIndex: Integer;
    FSemanticAction: string;
    FAttributes: string;
    fsaMinIndent: Integer;
    procedure SetProductionName(const Value: string);
    procedure SetProdIndex(const Value: Integer);
    procedure SetSemanticAction(const Value: string);
    procedure SetAttributes(const Value: string);
    function GetSemanticWhiteSpace(const LineText: string;
      const iStart: Integer): string;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddPath(const ProdBody: string): TCocoProductionPath;
    function GetLastPath: TCocoProductionPath;
    procedure RemovePath(const PathIdx: Integer);
    procedure ClearAllPaths;
    function GetATG: string;
    function GetPathCount: Integer;
    function GetPath(const idx: Integer): TCocoProductionPath;
    function GetPathsATG: string;
    property ProdIndex: Integer read FProdIndex write SetProdIndex;
  published
    { Published declarations }
    property ProductionName: string read FProductionName write SetProductionName;
    property SemanticAction: string read FSemanticAction write SetSemanticAction;
    property Attributes: string read FAttributes write SetAttributes;
    property saMinIndent: Integer read fsaMinIndent write fsaMinIndent;
  end;

implementation

{ TCocoProduction }

function TCocoProduction.AddPath(const ProdBody: string): TCocoProductionPath;
var
  p: TCocoProductionPath;
begin
  p := TCocoProductionPath.Create(Self);
  P.PathName := 'Path' + IntToStr(FProductionPaths.Count);
  P.PathBody := ProdBody;
  FProductionPaths.Add(p);
  Result := P;
end;

procedure TCocoProduction.ClearAllPaths;
var
  idx: integer;
  p: TCocoProductionPath;
begin
  for idx := FProductionPaths.Count - 1 downto 0 do
  begin
    P := FProductionPaths[idx];
    FProductionPaths.Delete(idx);
    if Assigned(P) then
      P.Free;
  end;
end;


constructor TCocoProduction.Create(AOwner: TComponent);
begin
  inherited;
  FProductionPaths := TList.Create;
end;

destructor TCocoProduction.Destroy;
begin
  FreeObjectList(FProductionPaths);
  inherited;
end;

function TCocoProduction.GetATG: string;
Var S : String;
begin
  if (FAttributes = '') then
    Result := FProductionName
  else
    Result := FProductionName + ' <' + FAttributes + '>';

  //+ ' =' + CRLF;

  if (FSemanticAction = '') then
    Result := Result + ' =' + CRLF
  else
    Result := Result + #13#10 + GetSemanticWhiteSpace('', 0) + #13#10 +
      ' =' + CRLF;
  S := GetPathsATG;
   S := '        '+ReplaceAll(S,#13#10,#13#10'        ');
  Result := Result + S;
  Result := Result + CRLF + '        .' + CRLF;
end;

function TCocoProduction.GetLastPath: TCocoProductionPath;
begin
  if (FProductionPaths.Count > 0) then
    Result := FProductionPaths[FProductionPaths.Count - 1]
  else
    Result := nil;
end;

function TCocoProduction.GetPath(const idx: Integer): TCocoProductionPath;
begin
  Assert(idx < FProductionPaths.Count, 'Bad index of production path.');
  Result := FProductionPaths[idx];
end;

function TCocoProduction.GetPathCount: Integer;
begin
  Result := FProductionPaths.Count;
end;

function TCocoProduction.GetPathsATG: string;
var
  idx: integer;
  P: TCocoProductionPath;
begin
  for idx := 0 to FProductionPaths.Count - 1 do
  begin
    p := FProductionPaths[idx];
    P.saMinIndent := fsaMinIndent;
    if (idx = 0) then
      Result := Result + P.GetATG
    else
      Result := Result + '        | ' + CRLF + P.GetATG;
  end;
end;


procedure TCocoProduction.RemovePath(const PathIdx: Integer);
var
  p: TCocoProductionPath;
begin
  P := FProductionPaths[PathIdx];
  FProductionPaths.Delete(PathIdx);
  if Assigned(P) then
    P.Free;
end;


procedure TCocoProduction.SetAttributes(const Value: string);
begin
  FAttributes := Value;
end;

procedure TCocoProduction.SetProdIndex(const Value: Integer);
begin
  FProdIndex := Value;
end;

procedure TCocoProduction.SetProductionName(const Value: string);
begin
  FProductionName := Value;
end;

procedure TCocoProduction.SetSemanticAction(const Value: string);
begin
  FSemanticAction := Value;
end;

function TCocoProduction.GetSemanticWhiteSpace(const LineText: string;
  const iStart: Integer): string;
var 
  S: string;
begin
  if (Trim(FSemanticAction) = '') then
    Result := ''
  else
  begin
    if FSemanticAction[1] = '.' then system.Delete(FSemanticAction, 1, 1);
    ExecuteSemActionFormatterParser('(. ' + FSemanticAction + '.)', 15, 0, S);
    Result := S
  end;
end;
//var
//  S, SOC: string;
//  idx, InitialOffset: integer;
//  // being lazy? - using a string list to break the semantic text into lines
//  // and to provide some manner of pascal source formatting - just to make the ATG
//  // file a little bit cleaner.
//  SL: TStringList;
//begin
//  Result := '';
//  if (FSemanticAction <> '') then
//  begin
//    S := FSemanticAction;
//    if (Length(LineTExt) > 0) then
//      InitialOffset := 57 - iStart - Length(LineText)
//    else
//      InitialOffset := 57 - iStart;
//    SOC := StringOfChar(' ', InitialOffset);
//
//    SL := TStringList.Create;
//    S := ReplaceAll(S, ';', ';'+#13#10);
//    StringToStringList(S, #13#10, SL);
//    for idx := Sl.Count - 1 downto 0 do
//    begin
//      SL[idx] := Trim(SL[idx]);
//      if (SL[idx] = '') then SL.Delete(idx)
//      else
//      begin
//        if (idx > 0) then
//          SL[idx] := StringOfChar(' ', 60) + SL[idx];
//      end;
//    end;
//    Result := SOC + '(. ' + Trim(SL.Text) + ' .)';
//    SL.Free;
//  end;
//end;


end.
