{-----------------------------------------------------------------------------
 Unit Name: CocoTokens
 Author:     pat connors
Purpose: Container for GUI-model of ATG File - a list of Tokens
-----------------------------------------------------------------------------}


unit CocoTokens;

interface

uses
  Windows, Messages, SysUtils, Classes, pbStrLib,
  CocoToken, jclSysUtils;

type
  TCocoTokens = class(TComponent)
  private
    FTokenText: TStringList;
    FTokenList: TList;
    FIsEditing: boolean;
    FEditingToken: string;
    procedure SetTokenText(const Value: TStringList);
    function GetTokenCount: Integer;
    procedure SetIsEditing(const Value: boolean);
    procedure SetEditingToken(const Value: string);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetATG: string;
    function AddTokenDefinition(const TokenDef: string): TCocoToken;
    function GetTokenDefinition(const TokenTitle: string): TCocoToken;
    function GetTokenByIndex(const idx: Integer): TCocoToken;
  published
    property TokenText: TStringList read FTokenText write SetTokenText;
    property TokenCount: Integer read GetTokenCount;
    property IsEditing: boolean read FIsEditing write SetIsEditing;
    property EditingToken: string read FEditingToken write SetEditingToken;
  end;

implementation

{ TCocoTokens }

function TCocoTokens.AddTokenDefinition(const TokenDef: string): TCocoToken;
begin
  //   FTokenText.Add(Trim(TokenDef));
  Result := TCocoToken.Create(Self);
  FTokenList.Add(Result);
end;

constructor TCocoTokens.Create(AOwner: TComponent);
begin
  inherited;
  { TODO : Kill off the TokenText : StringList - replaced by the TokenList :TList }
  FTokenText := TStringList.Create;
  FTokenList := TList.Create;
end;

destructor TCocoTokens.Destroy;
begin
  FTokenText.Free;
  FreeObjectList(FTokenList);
  inherited;
end;

function TCocoTokens.GetATG: string;
var
  idx: integer;
  ct: TCocoToken;
begin
  if not fIsEditing then
  begin
    Result := 'TOKENS' + CRLF;
    for idx := 0 to fTokenList.Count - 1 do
    begin
      ct := fTokenList[idx];
      if ct.Symbol <> FEditingToken then
        Result := Result + ct.GetATG
      else
        Result := Result + ModTextTag + CRLF;
    end;
  end
  else
  begin
    // Editing entire Token Section
    Result := 'TOKENS' + CRLF + ModTextTag + CRLF;
  end;
end;

function TCocoTokens.GetTokenByIndex(const idx: Integer): TCocoToken;
begin
  Result := fTokenList[idx];
end;

function TCocoTokens.GetTokenCount: Integer;
begin
  Result := fTokenList.Count;
end;

function TCocoTokens.GetTokenDefinition(const TokenTitle: string): TCocoToken;
var
  idx: integer;
  ct: TCocoToken;
begin
  Result := nil;
  for idx := 0 to fTokenList.Count - 1 do
  begin
    ct := fTokenList[idx];
    if (ct.Symbol = TokenTitle) then
    begin
      Result := ct;
      Exit;
    end;
  end;
end;


procedure TCocoTokens.SetEditingToken(const Value: string);
begin
  FEditingToken := Value;
end;

procedure TCocoTokens.SetIsEditing(const Value: boolean);
begin
  FIsEditing := Value;
end;

procedure TCocoTokens.SetTokenText(const Value: TStringList);
begin
  FTokenText.Assign(Value);
end;

end.
