{-----------------------------------------------------------------------------
 Unit Name: pbStrLib
 Author:    Pat Connors
 Purpose:
  String Routines for ParserBuilder - code taken from all over the place.
-----------------------------------------------------------------------------}


unit pbStrLib;

interface

uses
  Windows, Messages, SysUtils, Forms, Variants, Classes, FileUtil;


const
  TAB = #09;
  _LF = #10;
  _CR = #13;
  _EF = #0;
  EF = #0;
  _EL = _CR;
  LineEnds: array[0..2] of char = (#13, #10, #0);
  CRLF = #13#10;
  ModTextTag = '<!--Modified Text Here-->';


  // Text-File Stream function
procedure pbWriteLn(const f: TStringStream; const S: string);
procedure pbWrite(const f: TStringStream; const S: string);
procedure pbRead(const f: TStringStream; var S: string); overload;
procedure pbRead(const f: TStringStream; var ch: char); overload;
function LoadStringStream(const Filename: string): TStringStream;
procedure SaveStringStream(SS: TStringStream; const Filename: string);
procedure SaveString(S, FileName: string);
procedure LoadString(var S: string; FileName: string);
function StrExtract(S: string; Column: Integer; Separator: string): string;
function HeaderLine: string;
function ReplaceFirst(MainStr, KillStr, InsStr: string): string;
function ReplaceAll(MainStr, KillStr, InsStr: string): string;
function CaseInsensitiveReplaceAll(MainStr, KillStr, InsStr: string): string;
function KillFinalDelim(S, Delim: string): string;
function strColumns(S: string; Separator: string): Integer;
function GetIndexTitle(const idx: Integer): string;
procedure StringToStringList(Str, Separator: string; SL: TStrings);
function StringListToString(SL: TStrings; Separator: string; Ending: boolean): string;
  //procedure WriteStringToStream(const S: string; Strm: TStream);
  //function ReadStringFromStream(Strm: TStream): string;




implementation

function strColumns(S: string; Separator: string): Integer;
var
  Start, idx: integer;
  SwapOut: string;
begin
  // Assume everything except a blank string has at least one column
  if (S = '') then
  begin
    Result := 0;
    Exit;
  end;
  Result := 1;
  Start := Pos(Separator, S);
  Swapout := Separator;
  for idx := 1 to length(Separator) do
    SwapOut[idx] := char(Ord(SwapOut[idx]) + 1);
  while Start > 0 do
  begin
    Inc(Result);
    S := StringReplace(S, Separator, SwapOut, []);
    Start := Pos(Separator, S);
  end;
end;


function StrExtract(S: string; Column: Integer; Separator: string): string;
var
  idx, p: integer;
begin
  Result := S;
  if (S = '') then Exit;
  if (Separator = '') then Exit;
  if (Separator = s) then
  begin
    Result := '';
    Exit;
  end;

  idx := 1;
  // kill ignored sections
  p := Pos(Separator, S);
  while (idx < Column) and (P > 0) do
  begin
    S := Copy(S, P + Length(Separator), Length(S));
    p := Pos(Separator, S);
    inc(idx);
  end;

  // Kill next separator onwards;
  p := Pos(Separator, S);
  if (P > 0) then
    S := Copy(S, 1, p - 1);
  Result := S;
end;


procedure pbWriteLn(const f: TStringStream; const S: string);
begin
  pbWrite(f, s + CRLF);
end;

procedure pbWrite(const f: TStringStream; const S: string);
var 
  i: integer;
begin
  i := length(s);
  if I > 0 then
    f.WriteBuffer(s[1], i);
end;


procedure pbRead(const f: TStringStream; var S: string);
begin
  S := f.ReadString(1)
end;

procedure pbRead(const f: TStringStream; var ch: char);
var 
  S: string;
begin
  S := f.ReadString(1);
  if (S <> '') then
    ch := s[1]
  else
    CH := #0;
end;



function LoadStringStream(const Filename: string): TStringStream;
var 
  S: string;
begin
  LoadString(S, Filename);
  Result := TStringStream.Create(S);
end;

procedure SaveStringStream(SS: TStringStream; const Filename: string);
var 
  S: string;
begin
  S := SS.DataString;
  SaveString(S, Filename);
end;

procedure SaveString(S, FileName: string);
begin
  with TFileStream.Create(FileName, fmCreate) do
    try
      Write(Pointer(S)^, Length(S));
    finally
      Free;
    end;
end;

procedure LoadString(var S: string; FileName: string);
begin
  with TFileStream.Create(FileName,
    fmOpenRead or fmShareDenyWrite) do 
  begin
    try
      SetLength(S, Size);
      Read(Pointer(S)^, Size);
    except
      S := '';  // Deallocates memory
      Free;
      raise;
    end;
    Free;
  end;
end;

function HeaderLine: string;
var 
  idx: integer;
begin
  SetLength(Result, 78);
  Result := '{';
  for idx := 38 downto 0 do
    Result := Result + '+-';
  Result := Result + '}';
end;

function KillFinalDelim(S, Delim: string): string;
var 
  i, j: integer;
  S1: string;
begin
  i := Length(Delim);
  j := Length(S) - i;
  S1 := Copy(S, j + 1, Length(S));
  if (S1 = Delim) then
    S := Copy(S, 1, j);
  Result := S;
end;


function ReplaceFirst(MainStr, KillStr, InsStr: string): string;
begin
  Result := StringReplace(MainStr, KillStr, InsStr, []);
end;

function ReplaceAll(MainStr, KillStr, InsStr: string): string;
begin
  Result := StringReplace(MainStr, KillStr, InsStr, [rfReplaceAll]);
end;

function CaseInsensitiveReplaceAll(MainStr, KillStr, InsStr: string): string;
begin
  Result := StringReplace(MainStr, KillStr, InsStr, [rfReplaceAll, rfIgnoreCase]);
end;

function GetIndexTitle(const idx: Integer): string;
begin
  case idx of
    0: 
    begin  
      Result := 'Zeroth'; 
    end;
    1: 
    begin  
      Result := 'First'; 
    end;
    2: 
    begin  
      Result := 'Second'; 
    end;
    3: 
    begin  
      Result := 'Third'; 
    end;
    4: 
    begin  
      Result := 'Fourth'; 
    end;
    5: 
    begin  
      Result := 'Fifth'; 
    end;
    6: 
    begin  
      Result := 'Sixth'; 
    end;
    7: 
    begin  
      Result := 'Seventh'; 
    end;
    8: 
    begin  
      Result := 'Eigth'; 
    end;
    9: 
    begin  
      Result := 'Ninth'; 
    end;
    10: 
    begin  
      Result := 'Tenth'; 
    end;
  end; //Case
end;

procedure StringToStringList(Str, Separator: string;
  SL: TStrings);
var
  i, NumCols: integer;
begin
  Assert(SL <> nil, 'Dead StringList passed to StringToStringList Method.');
  SL.Clear;
  NumCols := StrColumns(Str, Separator);
  for i := 1 to NumCols do
    SL.Add(StrExtract(Str, i, Separator));
end;


function StringListToString(SL: TStrings; Separator: string; Ending: boolean): string;
var
  K: integer;
  aString: string;
begin
  Result := '';
  for K := 0 to SL.Count - 1 do
  begin
    aString := SL[K];
    Result := Result + aString + Separator;
  end;
  if not Ending then
    Result := Copy(Result, 1, Length(Result) - 1);
end;



end.
