{-----------------------------------------------------------------------------
 Unit Name: EditStreamCallbackHandler
 Author:     pat connors
 Purpose: Utility Class to assist in the Syntax Highlighting of the RichEdit.
          Based on code stolen from somewhere (maybe Peter B from TeamB?)

 Note: The whole Syntax Highlighting thing going on here is pretty dodgy -
       an experiment that unfortunately got used in ParserBuilder.
-----------------------------------------------------------------------------}


unit EditStreamCallbackHandler;

interface

uses
  Windows, Messages, SysUtils, Classes, RichEdit, ComCtrls, Dialogs, pbSHLParser,
  pbStrLib;

type
  TEditStreamCallbackHandler = class(TObject)
  private
    EditStream2: TEditStream;
    TextType: longint;
    Position, SelLen: longint;
    StreamInfo: TRichEditStreamInfo;
    Converter: TConversion;
    S: TStringStream;
    TempString: string;

    FRichEdit: TRichEdit;
    FAdjustment: Integer;
    FText: string;
    FSemActionCOlor: Integer;
    procedure SeTCustomRichEdit(const Value: TRichEdit);
    procedure SetAdjustment(const Value: Integer);
    procedure SetText(const Value: string);
    procedure SetSemActionCOlor(const Value: Integer);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    fpbSHLParser: TpbSHLParser;
    EditStream: TEditStream;
    constructor Create;
    destructor Destroy; override;
    procedure ApplyHighlight;
    procedure Init;
    procedure CleanString;
    function TopAndTailWithRTF(const S: string): string;
  published
    { Published declarations }
    property RichEdit: TRichEdit read FRichEdit write SetCustomRichEdit;
    property Adjustment: Integer read FAdjustment write SetAdjustment;
    property Text: string read FText write SetText;
    property SemActionCOlor: Integer read FSemActionCOlor write SetSemActionCOlor;
  end;


implementation

uses pbSHLScanner, StdCtrls;

function AdjustLineBreaks(Dest, Source: PChar): Integer; assembler;
asm
        PUSH    ESI
        PUSH    EDI
        MOV     EDI,EAX
        MOV     ESI,EDX
        MOV     EDX,EAX
        CLD
@@1:    LODSB
@@2:    OR      AL,AL
        JE      @@4
        CMP     AL,0AH
        JE      @@3
        STOSB
        CMP     AL,0DH
        JNE     @@1
        MOV     AL,0AH
        STOSB
        LODSB
        CMP     AL,0AH
        JE      @@1
        JMP     @@2
@@3:    MOV     EAX,0A0DH
        STOSW
        JMP     @@1
@@4:    STOSB
        LEA     EAX,[EDI-1]
        SUB     EAX,EDX
        POP     EDI
        POP     ESI
end;



function StreamSave(dwCookie: longint; pbBuff: PByte;
  cb: longint; var pbb: longint): longint; stdcall;
var
  StreamInfo: PRichEditStreamInfo;
begin
  Result := NoError;
  StreamInfo := PRichEditStreamInfo(Pointer(dwCookie));
  try
    pbb := 0;
    if StreamInfo^.Converter <> nil then
      pbb := StreamInfo^.Converter.ConvertWriteStream(StreamInfo^.Stream,
        PChar(pbBuff), cb);
  except
    Result := 2; //WriteError;
  end;
end;

function StreamLoad(dwCookie: longint; pbBuff: PByte;
  cb: longint; var pbb: longint): longint; stdcall;
var
  Buffer, pBuff: PChar;
  StreamInfo: PRichEditStreamInfo;
begin
  Result := NoError;
  StreamInfo := PRichEditStreamInfo(Pointer(dwCookie));
  Buffer := StrAlloc(cb + 1);
  try
    cb := cb div 2;
    pbb := 0;
    pBuff := Buffer + cb;
    try
      if StreamInfo^.Converter <> nil then
        pbb := StreamInfo^.Converter.ConvertReadStream(StreamInfo^.Stream, pBuff, cb);
      if pbb > 0 then
      begin
        pBuff[pbb] := #0;
        if pBuff[pbb - 1] = #13 then pBuff[pbb - 1] := #0;
        pbb := AdjustLineBreaks(Buffer, pBuff);
        Move(Buffer^, pbBuff^, pbb);
      end;
    except
      Result := 1; //ReadError;
    end;
  finally
    StrDispose(Buffer);
  end;
end;

{ TEditStreamCallbackHandler }

procedure TEditStreamCallbackHandler.ApplyHighlight;
begin
  if (fRichEdit = nil) then
    Exit;
  Init;
  S.Size := 0;
  try
    Position := fRichEdit.SelStart;;
    SelLen := fRichEdit.SelLength;
    //    fRichEdit.Clear;

    // Trying a different approach - now adding an RTF header to the plain text
    // and then parsing that to add the formatting (bold only)
    // - rather than mucking with the rtf itself. The problem is the difference in
    // the RichEdit RTF between win2000 and win98.

    //    SendMessage(RichEdit.Handle, EM_STREAMOUT, TextType, longint(@EditStream));
    //    if EditStream.dwError <> 0 then
    //      raise EOutOfResources.Create('Text retrieval failed.');
    //    TempString := S.DataString;
    //    CleanString;
    FText := ReplaceAll(FText, '\', '\\');
    FText := ReplaceAll(FText, '{', '\{');
    FText := ReplaceAll(FText, '}', '\}');
    FText := ReplaceAll(FText, #13#10, #13#10 + '\par ');


    fpbSHLParser.fFontSize := RichEdit.DefAttributes.Size;
    fpbSHLParser.SourceString := FText;
    fpbSHLParser.SemFontIdx := FSemActionCOlor;
    fpbSHLParser.Parse;
    S.Size := 0;
    //    SaveString(fpbSHLParser.lst.DataString,'C:\temp\s.txt');
    S.WriteString(TopAndTailWithRTF(fpbSHLParser.lst.DataString));
    //    SaveString(TopAndTailWithRTF(fpbSHLParser.lst.DataString),'C:\temp\stt.txt');
    S.Position := 0;
    SendMessage(RichEdit.Handle, EM_STREAMIN, TextType, longint(@EditStream2));
    if (EditStream.dwError <> 0) then
    begin
      SendMessage(RichEdit.Handle, EM_STREAMIN, TextType, longint(@EditStream2));
      if EditStream.dwError <> 0 then
        raise EOutOfResources.Create('Text load failed.');
    end;
  finally
    fRichEdit.SelStart := Position;
    //   fRichEdit.Lines.EndUpdate;
  end;
end;

procedure TEditStreamCallbackHandler.CleanString;
begin
  TempString :=
    '{\rtf1\ansi\deff0\deftab720' +
    '{\fonttbl' +
    '{\f0\fswiss MS Sans Serif;}' +
    '{\f1\froman\fcharset2 Symbol;}' +
    '{\f2\fnil\fcharset1 ' + RichEdit.DefAttributes.Name + ';}' +
    '{\f3\fswiss\fcharset1 Tahoma;}}' +
    '{\colortbl\red0\green0\blue0;}' +
    '\deflang1033\pard\plain\f2\fs' + IntToStr(RichEdit.DefAttributes.Size * 2) + ' ' +
    fText + '}';
  fpbSHLParser.fFontSize := RichEdit.DefAttributes.Size;
end;

constructor TEditStreamCallbackHandler.Create;
begin
  inherited;
  fpbSHLParser := TpbSHLParser.Create;
end;

destructor TEditStreamCallbackHandler.Destroy;
begin
  fpbSHLParser.Free;
  Converter.Free;
  S.Free;
  inherited;
end;

procedure TEditStreamCallbackHandler.Init;
begin
  if (fRichEdit = nil) or (Converter = nil) then
  begin
    Converter := RichEdit.DefaultConverter.Create;
    S := TStringStream.Create('');
    StreamInfo.Stream := S;
    StreamInfo.Converter := Converter;
    with EditStream do
    begin
      dwCookie := longint(Pointer(@StreamInfo));
      pfnCallBack := @StreamSave;
      dwError := 0;
    end;

    with EditStream2 do
    begin
      dwCookie := longint(Pointer(@StreamInfo));
      pfnCallBack := @StreamLoad;
      dwError := 0;
    end;
    TextType := SF_RTF;
  end;
end;

procedure TEditStreamCallbackHandler.SetAdjustment(const Value: Integer);
begin
  FAdjustment := Value;
end;

procedure TEditStreamCallbackHandler.SeTCustomRichEdit(const Value: TRichEdit);
begin
  FRichEdit := Value;
  if (Value <> nil) then
    Init;
end;

procedure TEditStreamCallbackHandler.SetSemActionCOlor(const Value: Integer);
begin
  FSemActionCOlor := Value;
end;

procedure TEditStreamCallbackHandler.SetText(const Value: string);
begin
  FText := Value;
end;


function TEditStreamCallbackHandler.TopAndTailWithRTF(const S: string): string;
begin
  Result :=
    '{\rtf1\ansi\deff0\deftab720' +
    '{\fonttbl' +
    '{\f0\fswiss MS Sans Serif;}' +
    '{\f1\froman\fcharset2 Symbol;}' +
    '{\f2\fnil\fcharset1 ' + RichEdit.DefAttributes.Name + ';}' +
    '{\f3\fswiss\fcharset1 Tahoma;}}' +
    '{\colortbl ;\red255\green0\blue0;\red0\green128\blue0;\red0\green0\blue255;\red192\green192\blue192;}' +
    '\deflang1033\pard\plain\f2\fs' + IntToStr(RichEdit.DefAttributes.Size * 2) + ' ' +
    S + '}';
end;

end.
